/*
 * LABANscript project drawer (code file)
 *
 * Written and Copyright 1999 by David E. Down the Faery Monarch, Gonksoftware
 * This software is released under the GNU GPL version 2.
 *
 * See the file "COPYING" for software license details.
 */

#include "def.h"
#include "gui.h"
#include "mabanol.h"
#include "labanscr.h"
#include "laban_pd.h"
#include "symbols.h"

/* Draw the LABAN stave on a chosen bitmap */
void laban_proj_drawer (BITMAP *bitmap, int handle,
                        LABAN_STRUCT *labandata,
                        int start, int end, /* Section of stave to update */
                        int being) { /* The row number of the script */

 int c, cc, x, y, y_, i, yy, ii, z, zz, yl, md,
     ct, cb,                                    /* Bitmap clippers */
     vr = labandata->labanator.vertspc,         /* Number of vertical rows */
     vs = labandata->labanator.vertsize,        /* Vertical size (Height of bitmap) */
     fl = labandata->labanator.farleft,         /* Far left of the stave */
     fr = labandata->labanator.farrght,         /* Far right of the stave */
     mos = labandata->labanator.stave_mid,      /* Middle of stave */
     sl = labandata->labanator.leftline,        /* Left Line */
     sl_ii = labandata->labanator.leftline_ii,  /* Left Line for Part Two */
     sm = labandata->labanator.midline,         /* Middle Line */
     sm_ii = labandata->labanator.midline_ii,   /* Middle Line for Part Two */
     sr = labandata->labanator.rightline,       /* Right Line */
     sr_ii = labandata->labanator.rightline_ii; /* Right Line for Part Two */

  /* Get window bitmap (prepare_to_draw must called first) */
    if (handle >= 0) {
     prepare_to_draw_win (handle);
     if (!bitmap) bitmap = window_bitmap(handle);
    }
  /* Ensure that the glyphs are in screen range */
   if ((start < 0 && end < 0) || (start > vr && end > vr)) return;
   if (start < 0) start = 0;
   if (end > vr) end = vr;

  /* Do the Background */
   y = ((vs-1) * (labandata->options.read_dir ? start : vr - start) / vr);
   y_ = ((vs-1) * (labandata->options.read_dir ? end : vr - end) / vr);
   ct = bitmap->ct; cb = bitmap->cb;
   bitmap->ct = (y < y_ ? y : y_); bitmap->cb = (y > y_ ? y : y_);
   rectfill (bitmap, fl, y , fr, y_, labandata->hs_labanbkgrd);

  /* Draw the LABAN grid */
   if (labandata->options.grid_mode) {
    drawing_mode (DRAW_MODE_MASKED_PATTERN, laban_dline, 0, 0);
     for (c = start; c <= end; c++) {
      y = ((vs-1) * (labandata->options.read_dir ? c : vr - c) / vr);
      hline (bitmap, fl, y, fr, labandata->hs_labangrid);
     }
    y = ((vs-1) * (labandata->options.read_dir ? start : vr - start) / vr);
    y_ = ((vs-1) * (labandata->options.read_dir ? end : vr - end) / vr);
     for (c = 0; c < 17; c++) {
      vline (bitmap, labandata->labanator.stavex[c], y, y_, labandata->hs_labangrid);
     }
    solid_mode ();
   } /* if */

  /* Draw the LABAN stave */
   for (c = start, ii = being; c < end; c++, ii++) {
    /* Take reading direction into consideration */
     if (ii >= labandata->maxpanto) break; /* Do not want to crash the program */
      y = ((vs-1) * (labandata->options.read_dir ? c : vr - c) / vr);
      y_ = ((vs-1) * (labandata->options.read_dir ? c + 1 : vr - c - 1) / vr);
       if (ii) {
        vline(bitmap, sl, y, y_, labandata->hs_labanstave);
        vline(bitmap, sm, y, y_, labandata->hs_labanstave);
        vline(bitmap, sr, y, y_, labandata->hs_labanstave);
       }
    }
  /* Draw the LABAN glyphs */
   for (c = start, ii = being; c < end; c++, ii++) {
     if (ii > 0 && ii < labandata->maxpanto) { /* There is nothing in row zero */
      for (cc = 0; cc < 16; cc++) { /* Print the symbols */
       z  = labandata->panto[ii].stave.symbol[cc];
       zz = labandata->panto[ii].stave.shader[cc];
       md = labandata->panto[ii].stave.modifier[cc];
       i  = labandata->panto[ii].stave.length[cc];

        if (z == 0xfe && c == start) { /* Continuation of a Glyph */
        /* Find backtrack data */
         yl = ii - i - 1;
         yy = c - i - 1;
        /* Get the backtracked glyph */
         z  = labandata->panto[yl].stave.symbol[cc];
         zz = labandata->panto[yl].stave.shader[cc];
         md = labandata->panto[yl].stave.modifier[cc];
         i  = labandata->panto[yl].stave.length[cc];
        /* Render the glyph */
         y = (labandata->options.read_dir ? (vs-1) * yy / vr : (vs-1) * (vr-yy) / vr);
         y_ = (labandata->options.read_dir ? (vs-1) * (yy + i + 1) / vr : (vs-1) * (vr-yy-i-1) / vr);
         draw_gylph (bitmap, labandata->labanator.stavex[cc], y,
                     labandata->labanator.stavex[cc+1], y_, z, zz & 0x07, 0,
                     labandata);
        } else if (z < 0x30) { /* The Polygon and Line Glyphs (0x00 to 0x2F) */
         y = (labandata->options.read_dir ? (vs-1) * c / vr : (vs-1) * (vr-c) / vr);
         y_ = (labandata->options.read_dir ? (vs-1) * (c + i + 1) / vr : (vs-1) * (vr-c-i-1) / vr);
         draw_gylph (bitmap, labandata->labanator.stavex[cc], y,
                     labandata->labanator.stavex[cc+1], y_, z, zz & 0x07, 0,
                     labandata);
        } else if (z == 0x30) { /* The Space Glyphs (0x30) */
         y = (labandata->options.read_dir ? (vs-1) * c / vr : (vs-1) * (vr-c) / vr);
         y_ = (labandata->options.read_dir ? (vs-1) * (c + 1) / vr : (vs-1) * (vr-c-1) / vr);
         draw_gylph (bitmap, labandata->labanator.stavex[cc], y,
                     labandata->labanator.stavex[cc+1], y_, z, zz, md,
                     labandata);
        } /* if (z ... */
       } /* for */
      } /* if (ii > 0 ... */
    } /* for (c = ... */

  /* Draw the LABAN stave dividers */
   for (c = start, ii = being - 1; c <= end; c++, ii++) {
    if (ii >= labandata->maxpanto) break; /* Do not want to crash the program */
     y = ((vs-1) * (labandata->options.read_dir ? c : vr - c) / vr);
      if (ii >= 0) {
       switch (labandata->panto[ii].bar.type) {
         case 0x00: { hline(bitmap, sl, y, sr, labandata->hs_labanstave); break; }
         case 0x01: { hline(bitmap, sl, y - 1, sr, labandata->hs_labanstave);
                      hline(bitmap, sl, y + 1, sr, labandata->hs_labanstave); break; }
       /* Tick Mark */
         case 0xf0: { hline(bitmap, (sl / 6) + (sm * 5 / 6), y,
                            (sm * 5 / 6) + (sr / 6), labandata->hs_labanstave); break; }
        }
       } /* if (ii>=0) */
   } /* for */
  bitmap->ct = ct; bitmap->cb = cb;
   if (handle >= 0) {
    blit_sub_window (handle, fl, ct, fr-fl, cb - ct); /* Blit window */
   }
 }

