/*
 * LABANscript Applet Routines (Code File)
 *
 * Written and Copyright 1999 by David E. Down the Faery Monarch, Gonksoftware
 * This software is released under the GNU GPL version 2.
 *
 * See the file "COPYING" for software license details.
 */

#include <fcntl.h>
#include "def.h"
#include "gui.h"
#include "mabanol.h"
#include "labanscr.h"
#include "laban_pd.h"
#include "symbols.h"

/* LABANATOR Colour Scheme */
 char *labancolors [] =    {"Grid",
                            "Stave Lines"
                            "Part Divider",
                            "Stave Background"
                            "Pin Crater",
                            "Polygon Outline",
                            "Polygon Secondary",
                            "Space Glyphs"};

/*
 * Germanic Alphabet
 * Latin Alphabet
 */

#define LABAN_THREAD_RATE 0x4000
#define labanclrdefault (RGB [maxlabancolors]) \
                         {{192, 192, 0},       \
                          {0, 0, 128},         \
                          {192, 0, 0},         \
                          {255,255, 224},      \
                          {0,128,0},           \
                          {0,128,128},         \
                          {0,255,255},         \
                          {0,0,128},           \
                          {128,0,0}}

/* Numbers for the New Project Name Generator */
 int laban_count = 0;

/* LABAN option defaults */
 LABAN_OPTIONS laban_option_defaults = {TRUE,  /* Top-to-Bottom Reading Direction */
                                        TRUE,  /* Grid Mode */
                                        TRUE,  /* Show Crater Mode */
                                        0,     /* Dialect Mode */
                                        labanclrdefault}; /* Laban Colours */

/* For those POPups */
#define POPUP_PALETTE int
#define POPUP_POLYGON -1 /* Glyph Icons */
#define POPUP_SHADE   -2 /* Shade Icons */
#define POPUP_SS      -3 /* Space Glyph Basic Icons */
#define POPUP_SS_C    -4 /* Space Glyph Affixation Icon */
#define POPUP_DATAEND -9999

/* Polyglyph Palette */
 POPUP_PALETTE polyglyph_popup[] = {
   104, 71,

   POPUP_POLYGON, 23, 1,  0,  11, 23,
   POPUP_POLYGON, 20, 1,  24, 11, 23,
   POPUP_POLYGON, 18, 1,  48, 11, 23,

   POPUP_POLYGON, 13, 14, 0,  11, 23,
   POPUP_POLYGON, 15, 14, 24, 11, 23,
   POPUP_POLYGON, 21, 14, 48, 11, 23,

   POPUP_POLYGON, 2,  27, 0,  11, 23,
   POPUP_POLYGON, 1,  27, 24, 11, 23,
   POPUP_POLYGON, 8,  27, 48, 11, 23,

   POPUP_POLYGON, 3,  40, 0,  11, 23,
   POPUP_POLYGON, 0,  40, 24, 11, 23,
   POPUP_POLYGON, 9,  40, 48, 11, 23,

   POPUP_POLYGON, 4,  53, 0,  11, 23,
   POPUP_POLYGON, 7,  53, 24, 11, 23,
   POPUP_POLYGON, 11, 53, 48, 11, 23,

   POPUP_POLYGON, 5,  66, 0,  11, 23,
   POPUP_POLYGON, 6,  66, 24, 11, 23,
   POPUP_POLYGON, 12, 66, 48, 11, 23,

   POPUP_POLYGON, 14, 79, 0,  11, 23,
   POPUP_POLYGON, 16, 79, 24, 11, 23,
   POPUP_POLYGON, 22, 79, 48, 11, 23,

   POPUP_POLYGON, 24, 92, 0,  11, 23,
   POPUP_POLYGON, 19, 92, 24, 11, 23,
   POPUP_POLYGON, 17, 92, 48, 11, 23,

   POPUP_DATAEND
  };

/* Polyglyph Palette */
 POPUP_PALETTE polyshade_popup[] = {
   104, 71,

   POPUP_SHADE, 0 , 1,  0,  11, 23,
   POPUP_SHADE, 1 , 1,  24, 11, 23,
   POPUP_SHADE, 2 , 1,  48, 11, 23,

/* POPUP_SHADE, 3 , 14, 0,  11, 23, */ /* Not supported yet! */
   POPUP_SHADE, 4 , 14, 24, 11, 23,
   POPUP_SHADE, 5 , 14, 48, 11, 23,

   POPUP_DATAEND
  };

/* Polyglyph Palette */
 POPUP_PALETTE ss_popup[] = {
   104, 96,

   POPUP_SS, 0x00 , 1,  0,  11, 11, // POPUP_SS, 0x34 , 1,  24, 11, 11,
   POPUP_SS, 0x04 , 1,  12, 11, 11, // POPUP_SS, 0x38 , 1,  36, 11, 12,

   POPUP_SS, 0x08 , 13, 0,  11, 11, // POPUP_SS, 0x3C , 13, 24, 11, 12,
   POPUP_SS, 0x0c , 13, 12, 11, 11, // POPUP_SS, 0x40 , 13, 36, 11, 12,

   POPUP_SS, 0x10 , 25, 0,  11, 11, // POPUP_SS, 0x44,  25, 24, 11, 12,
   POPUP_SS, 0x14 , 25, 12, 11, 11, // POPUP_SS, 0x98,  25, 36, 11, 12,

   POPUP_SS, 0x18 , 37, 0,  11, 11, // POPUP_SS, 0x9C,  37, 24, 11, 12,
   POPUP_SS, 0x1c , 37, 12, 11, 11, // POPUP_SS, 0xa0,  37, 36, 11, 12,

   POPUP_DATAEND
  };

/* Space Glyph Combination Set */
 POPUP_PALETTE ss_popup_c [] = {
   93, 45,
   POPUP_SS_C , 0x00, 1,  0, 22, 22,   POPUP_SS_C , 0x01,  24, 0, 22, 22,
   POPUP_SS_C , 0x02, 47, 0, 22, 22,   POPUP_SS_C , 0x03,  70, 0, 22, 22,

   POPUP_SS_C , 0x80, 1,  23, 22, 22,  POPUP_SS_C , 0x81,  24, 23, 22, 22,
   POPUP_SS_C , 0x82, 47, 23, 22, 22,  POPUP_SS_C , 0x83,  70, 23, 22, 22,

   POPUP_DATAEND
  };

/* The drag system on the palette */
int popup_palette_drag (int handle, void *value) {
  LABAN_STRUCT *labandata = value;
  POPUP_PALETTE *pal = labandata->labanator.special;
  int select = labandata->labanator.popup_stx, newselect = -1, cx, cy,
      popup = labandata->labanator.popup, c = 2, i = 2, id, x, y, h, v, cde;
  bool flag = FALSE;
  BITMAP *bitmap = overlay_bitmap (popup);

  if (!get_mouse_on_overlay (popup, &cx, &cy)) { /* Mouse is off screen */
   cx -= 2; cy -= 2; /* Round the mouse cursor */
   /* Find the cursor */
    while ((id = pal[c]) != POPUP_DATAEND) {
     i = c; c++;
      switch (id) {
       case POPUP_SS_C:      /* A space glyph affixation */
       case POPUP_SS:        /* A space basic glyph */
       case POPUP_SHADE:     /* A shade glyph */
       case POPUP_POLYGON: { /* A polygon glyph */
        cde = pal[c++]; x = pal[c++]; y = pal[c++]; h = pal[c++]; v = pal[c++];
         if (cx >= x && cy >= y && cx < x + h && cy < y + v) {
          newselect = i; flag = TRUE;
         }
        break; }
       }
      if (flag) break;
     }
   }
  if (newselect == select) return 0; /* Already on selection, no update */
  /* Erase the old cursor */
   if ((c = select) >= 0) {
    c++; cde = pal[c++]; x = pal[c++]; y = pal[c++]; h= pal[c++]; v = pal[c];
    rect (bitmap, x + 1, y + 1, x + h + 2, y + v + 2, labandata->hs_labanbkgrd);
   }
  /* Display new cursor */
   if ((c = newselect) >=0) {
    c++; cde = pal[c++]; x = pal[c++]; y = pal[c++]; h= pal[c++]; v = pal[c];
    rect (bitmap, x + 1, y + 1, x + h + 2, y + v + 2, labandata->hs_labanstave);
   }
  /* Tally with new location */
  blit_tbmo (popup);
  labandata->labanator.popup_stx = newselect;
  return 0;
 }

/* Draw the LABAN pop-up palette
 * (returns the popup handle number)
 */
int popup_palette_draw (POPUP_PALETTE *pal, int handle, int param,
                        int secparam, int x_, int ya_, int yb_,
                        LABAN_STRUCT *labandata,
                        int (*proc)(int handle, void *value)) {
  int c = 0, id, x, y, h, v, cde, popup;
  BITMAP *bitmap;

  /* Get the window size */
   x = pal [c++]; y = pal [c++];
  /* Create the pop-up window */
   if ((popup = place_overlay (NULL, handle, x_ - ((x + 4) >> 1), ya_, yb_,
                x + 4, y + 4, labandata->options.read_dir, FALSE, FALSE)) < 0) return -1;
  /* Get bitmap pointer */
   bitmap = overlay_bitmap (popup);
  /* Draw the palette outlay */
   rect     (bitmap, 0, 0, x + 3, y + 3, labandata->hs_labangrid);
   rectfill (bitmap, 1, 1, x + 2, y + 2, labandata->hs_labanbkgrd);
  /* Draw the glyphs */
    while ((id = pal[c++]) != POPUP_DATAEND) {
      switch (id) {
       case POPUP_POLYGON: { /* A polygon glyph */
        cde = pal[c++]; x = pal[c++]; y = pal[c++]; h= pal[c++]; v = pal[c++];
        draw_gylph (bitmap, x + 2, y + 2, x + h + 1, y + v + 1, cde, param, 0, labandata);
        break; }
       case POPUP_SHADE:   { /* A shade glyph */
        cde = pal[c++]; x = pal[c++]; y = pal[c++]; h= pal[c++]; v = pal[c++];
        draw_gylph (bitmap, x + 2, y + 2, x + h + 1, y + v + 1, 7, cde, 0, labandata);
        break; }
       case POPUP_SS:      { /* A Spacing Glyph (code 0x30) */
        cde = pal[c++]; x = pal[c++]; y = pal[c++]; h= pal[c++]; v = pal[c++];
        draw_gylph (bitmap, x + 2, y + 2, x + h + 1, y + v + 1, 0x30, cde, param, labandata);
        break; }
       case POPUP_SS_C:    { /* The Space Glyph Affixation (code 0x30) */
        cde = ((pal[c++] & 0x83) | (param & 0x7c));
        x = pal[c++]; y = pal[c++]; h= pal[c++]; v = pal[c++];
        draw_gylph (bitmap, x + 2, y + 2, x + h + 1, y + v + 1, 0x30, cde, secparam, labandata);
        break; }
      }
     }
  /* Blit the overlay */
   labandata->labanator.special = pal;
   labandata->labanator.popup_stx = -1;
   blit_tbmo (labandata->labanator.popup = popup);
   win_dragstart (handle, popup_palette_drag, proc);
  /* Return with the handle */
   return popup;
 }

/* LABAN Attributive Window Close */
int laban_attributive_close (int handle, DIALOG_INFO *info, DIALOG_DATA *data) {
 LABAN_STRUCT *labandata = (LABAN_STRUCT *) info->value;

 /* Replace strings */
  string_replace(&labandata->attribution.title, data[1].field);
  string_replace(&labandata->attribution.bard, data[3].field);
  string_replace(&labandata->attribution.scriber, data[5].field);
  string_replace(&labandata->attribution.type, data[7].field);
  string_replace(&labandata->attribution.chapter, data[9].field);
  string_replace(&labandata->attribution.movement, data[11].field);
  string_replace(&labandata->attribution.part, data[13].field);
  string_replace(&labandata->attribution.description, data[15].field);
  string_replace(&labandata->attribution.copyright, data[17].field);
 /* Deallocate mallocs in the dialog fields */
  free (data[1]. field); /* Title */
  free (data[3]. field); /* BARD */
  free (data[5]. field); /* Notation Scriber */
  free (data[7]. field); /* Pantomime Type */
  free (data[9]. field); /* Chapter */
  free (data[11].field); /* Movement */
  free (data[13].field); /* Part */
  free (data[15].field); /* Description */
  free (data[17].field); /* Copyright */
  labandata->saved = FALSE; /* Attribution Data Changed, set saved flag to false */

 /* Close the dialog */
  return simple_dialog_close (handle, info, data);
 }

/* LABAN Attributive Window Close on Cancel */
int laban_attributive_cancel (int handle, DIALOG_INFO *info, DIALOG_DATA *data) {

 /* Deallocate mallocs in the dialog fields */
  free (data[1]. field); /* Title */
  free (data[3]. field); /* BARD */
  free (data[5]. field); /* Notation Scriber */
  free (data[7]. field); /* Pantomime Type */
  free (data[9]. field); /* Chapter */
  free (data[11].field); /* Movement */
  free (data[13].field); /* Part */
  free (data[15].field); /* Description */
  free (data[17].field); /* Copyright */

 /* Free Dialog Structure */
  return simple_dialog_close (handle, info, data);
 }

/* Pull out the Attributive Window */
void laban_attributive_info (int parent, LABAN_STRUCT *labandata) {
 DIALOG_INFO info [] = {
    {"Pantomime Attribution", 365, 244, WF_NO_OPTIONS, WIN_WITHFILE, labandata, labandata->filename}
   };
 DIALOG_DATA data [] = {
    {8, 8, 1, 0, 0, 0, clrsh_winfore, "Pantomime Title:", biosfont, NULL, 0},
    {8, 16, DIALOG_INPUT, 349, 12, 0, 42,
      strmalloc(labandata->attribution.title, 43), NULL, NULL, 0},
    {8, 30, 1, 0, 0, 0, clrsh_winfore, "Body Animation Routine Designer(s):", biosfont, NULL, 0},
    {8, 38, DIALOG_INPUT,349, 12, 0, 42,
      strmalloc(labandata->attribution.bard, 43), NULL, NULL, 0},
    {8, 52, 1, 0, 0, 0, clrsh_winfore, "Kinetographer(s):", biosfont, NULL, 0},
    {8, 60, DIALOG_INPUT,349, 12, 0, 42,
      strmalloc(labandata->attribution.scriber, 43), NULL, NULL, 0},
    {8, 74, 1, 0, 0, 0, clrsh_winfore, "Pantomime Type:", biosfont, NULL, 0},
    {8, 82, DIALOG_INPUT,349, 12, 0, 42,
      strmalloc(labandata->attribution.type, 43), NULL, NULL, 0},
    {8, 96, 1, 0, 0, 0, clrsh_winfore, "Chapter:", biosfont, NULL, 0},
    {8, 104, DIALOG_INPUT,349, 12, 0, 42,
      strmalloc(labandata->attribution.chapter, 43), NULL, NULL, 0},
    {8, 118, 1, 0, 0, 0, clrsh_winfore,"Routine:", biosfont, NULL, 0},
    {8, 126, DIALOG_INPUT,349, 12, 0, 42,
      strmalloc(labandata->attribution.movement, 43), NULL, NULL, 0},
    {8, 140,1, 0, 0, 0, clrsh_winfore, "For the actor(s) of:", biosfont, NULL, 0},
    {8, 148,DIALOG_INPUT,349, 12, 0, 42,
      strmalloc(labandata->attribution.part, 43), NULL, NULL, 0},
    {8, 162, 1, 0, 0, 0, clrsh_winfore, "Description:", biosfont, NULL, 0},
    {8, 170, DIALOG_INPUT,349, 12, 0, 42,
      strmalloc(labandata->attribution.description, 43), NULL, NULL, 0},
    {8, 184,1, 0, 0, 0, clrsh_winfore, "Copyright:", biosfont, NULL, 0},
    {8, 192,DIALOG_INPUT,349, 12, 0, 42,
      strmalloc(labandata->attribution.copyright, 43), NULL, NULL, 0},
    {237, 212, DIALOG_BUTTON, 64,24,0,0,"Cancel",NULL, laban_attributive_cancel, 'C'},
    {309, 212, DIALOG_BUTTON, 48,24,0,0,"Okay", NULL, laban_attributive_close, 'O'},
    {8, 220, 1, 0, 0, 0, clrsh_winfore, "LABAN Dialect:", biosfont, NULL},
    {8, 228, 1, 0, 0, 0, clrsh_winfore,
      (labandata->options.dialect == 0 ? "Rudolf von Laban's Original" :
      (labandata->options.dialect == 1 ? "Futher Developed Version": "Other")), biosfont, NULL},
    {0, 0, 0, 0, 0, 0, 0, NULL, NULL, laban_attributive_cancel, 0}
   };

  simple_dialog_open (parent, -5000, -5000, TRUE, info, data);
 }

/* Call up Laban Attributive Dialog */
int laban_attributive_dialog (int handle, DIALOG_INFO *info, DIALOG_DATA *data) {
  int parent = window_parent(handle);

  change_window_locks (TRUE, parent, -1); /* Lock the parent window(s) */
  laban_attributive_info (parent, info->value);
  return -1;
 }

/* Extend the LABAN pantomime (but does not do stave markings */
bool laban_proj_extend (LABAN_STRUCT *labandata, int newpantosize) {
  LABAN_PANTOMIME_SYMBOL *symbol;   /* Symbol */
  LABAN_PANTOMIME_LENGTH *length;   /* Length */
  LABAN_PANTOMIME_SHADER *shader;   /* Shader */
  LABAN_PANTOMIME_MODIFY *modifier; /* Modifier */
  LABAN_PANTOMIME_ALIGN  *align;    /* Modifier Aligment, center is 120 */
  LABAN_PANTOMIME        *panto;
  int cx = labandata->parts * labandata->width; /* Get stave width */

  if (newpantosize <= labandata->maxpanto) return FALSE; /* No need */
  if (!(panto = malloc(sizeof(LABAN_PANTOMIME) * newpantosize))) return TRUE; /* Could not allocate temporary space */
  memcpy (panto, labandata->panto, sizeof(LABAN_PANTOMIME) * newpantosize);
  free(labandata->panto);   /* Destroy old memory */
  labandata->panto = panto; /* Use copied memory */

  /* Extend the Stave */
   while (newpantosize > labandata->maxpanto) {
    if (symbol = malloc(sizeof(LABAN_PANTOMIME_SYMBOL)*cx)) {;
     memset(symbol, 0xff, sizeof(LABAN_PANTOMIME_SYMBOL)*cx); /* 0xff all the fields */
      if (length = malloc(sizeof(LABAN_PANTOMIME_LENGTH)*cx)) {
       if (shader  = malloc(sizeof(LABAN_PANTOMIME_SHADER)*cx)) {
        if (modifier = malloc(sizeof(LABAN_PANTOMIME_MODIFY)*cx)) {
         if (align = malloc(sizeof(LABAN_PANTOMIME_ALIGN)*cx)) {
          labandata->panto[labandata->maxpanto].stave.symbol   = symbol;
          labandata->panto[labandata->maxpanto].stave.length   = length;
          labandata->panto[labandata->maxpanto].stave.shader   = shader;
          labandata->panto[labandata->maxpanto].stave.modifier = modifier;
          labandata->panto[labandata->maxpanto].stave.align    = align;
          (labandata->maxpanto)++;
         } else { /* if (align = ... */
          free (modifier); free (shader); free (length); free(symbol); break;
         }
        } else { /* if (modifier = ... */
         free (shader); free (length); free(symbol); break;
        }
       } else { /* if (shader = .... */
        free (length); free(symbol); break;
       }
      } else { /* if (length = ... */
       free (symbol); break;
      }
    } /* if (symbol = ... */
   }

  /* Is the new stave too large */
   if (labandata->maxpanto < newpantosize) {
    labandata->panto = realloc(labandata->panto, sizeof(LABAN_PANTOMIME) *
                               labandata->maxpanto);
   }
  return FALSE;
 }

/* Extend the LABAN stave a-lot */
bool laban_proj_stave_extend (LABAN_STRUCT *labandata, int newpantosize) {
 int oldpantosize = labandata->maxpanto, i = oldpantosize, ii, bk_i, type,
     barsize, len, beats, size;

  if (laban_proj_extend (labandata, newpantosize)) return TRUE;
 /* Include the Stave Marks */
   while (i < labandata->maxpanto) {
    ii = bk_i = i-1;
    type = labandata->panto[ii].bar.type;
     if (type >= 0xf0) { /* Check for referal information */
      ii -= labandata->panto[ii].bar.size + 1;
      type = labandata->panto[ii].bar.type;
     }
    len = i - ii;
     if (type >= 0xe0 && type < 0xf0) { /* Untimed */
      labandata->panto[i].bar.type = 0xef;
     } else if (type < 0xe0) { /* Could be untimed */
      /* Get Bar size */
       if (barsize = beats = labandata->panto[ii].bar.beats & 0x3f) {
        barsize *= (size = labandata->panto[ii].bar.size & 0x3f);
       }
      /* Place the bar marking */
       if (!barsize) {
        labandata->panto[i].bar.type = 0xef; /* No ticker timing is permitted */
       } else if (len < barsize) {
        /* Can a tickmark be inserted? */
         if (!(len % size)) { /* Can be a tick be placed (unless disabled) */
          labandata->panto[i].bar.type = (labandata->panto[ii].bar.size & 0x80 ? 0xff : 0xf0);
         } else {
          labandata->panto[i].bar.type = 0xff;
         }
        /* Place backtrack information */
        labandata->panto[i].bar.size = len - 1;
       } else { /* Copy Bar Information */
        labandata->panto[i].bar = labandata->panto[ii].bar;
        labandata->panto[i].bar.type = 0x00;
       }
     }
    i++;
   }
  return FALSE;
 }

/* Configure Project's Notation Options */
void laban_notation_config (int parent, LABAN_STRUCT *labandata) {
 DIALOG_INFO info [] = {
    {"Notation Options", 275, 216, WF_NO_OPTIONS, WIN_WITHFILE, labandata, labandata->filename}
   };
 DIALOG_DATA data [] = {
    {137,  8, 2, 0 , 0 , 0 , clrsh_winfore,"Stave Reading Direction", biosfont, NULL},
    {8,  20, DIALOG_RADIO, 10, 10, (labandata->options.read_dir), 0, NULL, NULL, NULL, 'T'},
    {27, 22, 1, 0, 0, 0, clrsh_winfore, "Top to bottom", biosfont, NULL},
    {8,  32, DIALOG_RADIO, 10, 10, (!labandata->options.read_dir),0, NULL, NULL, NULL, 'B'},
    {27, 34, 1, 0, 0, 0, clrsh_winfore, "Bottom to top (Standard LABAN)", biosfont, NULL},
    {137, 52, 2, 0 , 0 , 0 , clrsh_winfore, "LABAN dialect type", biosfont, NULL},
    {8, 64, DIALOG_RADIO, 10, 10, (labandata->options.dialect == 0), 1, NULL, NULL, NULL, 'K'},
    {27,66, 1, 0, 0, 0, clrsh_winfore, "Rudolf von Laban's Original", biosfont, NULL},
    {8, 76, DIALOG_RADIO, 10, 10, (labandata->options.dialect == 1), 1, NULL, NULL, NULL, 'L'},
    {27,78, 1, 0, 0, 0, clrsh_winfore, "Futher Developed Version", biosfont,NULL},
    {8, 88, DIALOG_RADIO, 10, 10, (labandata->options.dialect > 1), 1, NULL, NULL, NULL, 'O'},
    {27, 90,1, 0, 0, 0, clrsh_winfore, "Other", biosfont, NULL},
    {137,108, 2, 0, 0, 0, clrsh_winfore, "Other Options", biosfont, NULL},
    {8,  120, DIALOG_CHECK, 10, 10, (labandata->options.grid_mode),0, NULL, NULL, NULL, 'G'},
    {27, 122, 1, 0, 0, 0, clrsh_winfore, "Enable the grid", biosfont, NULL},
    {8,  132, DIALOG_CHECK, 10, 10, (labandata->options.use_crater),0, NULL, NULL, NULL, 'C'},
    {27, 134, 1, 0, 0, 0, clrsh_winfore, "Show pin craters", biosfont, NULL},
    {163, 152, DIALOG_BUTTON, 104,24,0,0,"Get Default", NULL, NULL, 'G'},
    {27,  184, DIALOG_BUTTON, 112,24,0,0,"Make Default", NULL, NULL, 'M'},
    {147, 184, DIALOG_BUTTON, 64,24,0,0,"Cancel",NULL, simple_dialog_close, 'C'},
    {219, 184, DIALOG_BUTTON, 48,24,0,0,"Okay", NULL, NULL, 'O'},
    {0, 0, 0, 0, 0, 0, 0, NULL, NULL, simple_dialog_close, 0}
   };

  simple_dialog_open (parent, -5000, -5000, TRUE, info, data);
 }

/* LABAN notation options */
int laban_notation_options (int handle, DIALOG_INFO *info, DIALOG_DATA *data) {
  int parent = window_parent(handle);

  change_window_locks (TRUE, parent, -1); /* Lock the parent window(s) */
  laban_notation_config (parent, info->value);
  return -1;
 }

/* Save the stave options for the stave */
int laban_stave_save (int handle, DIALOG_INFO *info, DIALOG_DATA *data) {
  LABAN_STRUCT *labandata = (LABAN_STRUCT *) info->value;
  char flag = FALSE, warning = FALSE;

#ifdef DEBUG
   fprintf(stderr,"labanscr.c: laban_stave_save being executed: at %U\n", gui_timer);
#endif

   if (data[1].status != labandata->options.read_dir) {
#ifdef DEBUG
    fprintf(stderr,"labanscr.c: laban_stave_save changes status for reading direction at: %U\n", gui_timer);
#endif
    labandata->options.read_dir = data[1].status; flag = TRUE;
   }

   if (flag)
#ifdef DEBUG
    { fprintf(stderr,"labanscr.c: laban_stave_save has to redraw the window at: %U\n", gui_timer);
#endif
    win_redraw (window_parent(handle), TRUE); /* Only update if there is change */
#ifdef DEBUG
    }
#endif
  return simple_dialog_close (handle, info, data);
 }

void laban_stave_config (int parent, LABAN_STRUCT *labandata) {
 DIALOG_INFO info [] = {
    {"Stave Settings", 275, 148, WF_NO_OPTIONS, WIN_WITHFILE, labandata, labandata->filename}
   };
 DIALOG_DATA data [] = {
    {137,  8, 2, 0 , 0 , 0 , clrsh_winfore,"Stave Reading Direction", biosfont, NULL},
    {8,  20, DIALOG_RADIO, 10, 10, (labandata->options.read_dir), 0, NULL, NULL, NULL, 'T'},
    {27, 22, 1, 0, 0, 0, clrsh_winfore, "Top to bottom", biosfont, NULL},
    {8,  32, DIALOG_RADIO, 10, 10, (!labandata->options.read_dir),0, NULL, NULL, NULL, 'B'},
    {27, 34, 1, 0, 0, 0, clrsh_winfore, "Bottom to top (Standard LABAN)", biosfont, NULL},
    {137, 52, 2, 0 , 0 , 0 , clrsh_winfore, "Number of Parts", biosfont, NULL},
    {8,  64, DIALOG_RADIO, 10, 10, (labandata->parts == 1), 1, NULL, NULL, NULL, '1'},
    {27, 66, 1, 0, 0, 0, clrsh_winfore, "1 (one)", biosfont, NULL},
    {137,64, DIALOG_RADIO, 10, 10, (labandata->parts == 2), 1, NULL, NULL, NULL, '2'},
    {156,66, 1, 0, 0, 0, clrsh_winfore, "2 (two)", biosfont, NULL},
    {137, 84, 2, 0 , 0 , 0 , clrsh_winfore, "Size of the stave for each part", biosfont, NULL},
    {8, 96, DIALOG_RADIO, 10, 10, (labandata->width == 16), 2, NULL, NULL, NULL, 'T'},
    {27,98, 1, 0, 0, 0, clrsh_winfore, "Thin-16", biosfont, NULL},
    {137, 96, DIALOG_RADIO, 10, 10, (labandata->width == 24), 2, NULL, NULL, NULL, 'W'},
    {156,98, 1, 0, 0, 0, clrsh_winfore, "Wide-24", biosfont,NULL},
    {147, 116, DIALOG_BUTTON, 64,24,0,0,"Cancel",NULL, simple_dialog_close, 'C'},
    {219, 116, DIALOG_BUTTON, 48,24,0,0,"Okay", NULL, laban_stave_save, 'O'},
    {0, 0, 0, 0, 0, 0, 0, NULL, NULL, simple_dialog_close, 0}
   };

  simple_dialog_open (parent, -5000, -5000, TRUE, info, data);
 }

int laban_stave_settings (int handle, DIALOG_INFO *info, DIALOG_DATA *data) {
  int parent = window_parent(handle);

  change_window_locks(TRUE, parent, -1); /* Lock the parent window(s) */
  laban_stave_config (parent, info->value);
  return -1;
 }

/* LABAN project special */
int laban_proj_special (int handle, void *value) {
 LABAN_STRUCT *labandata = value;
 int x;

 DIALOG_INFO info [] = {
    {"Labanator Special", 192, 72, WF_NO_OPTIONS, WIN_WITHFILE, labandata, labandata->filename}};
 DIALOG_DATA data [] = {
    {0,  0, DIALOG_BUTTON,  192, 24, 0, 0, "Attribution", NULL, laban_attributive_dialog, 'A'},
    {0, 24, DIALOG_BUTTON,  192, 24, 0, 0, "Notation Options", NULL, laban_notation_options, 'O'},
    {0, 48, DIALOG_BUTTON,  192, 24, 0, 0, "Stave Settings", NULL, laban_stave_settings, 'S'},
    {0,  0, 0, 0, 0, 0, 0, NULL, NULL, simple_dialog_close}
   };

#ifdef DEBUG
 fprintf(stderr, "labanscr.c: laban_proj_special is being executed at: %u\n", gui_timer);
#endif
  if ((x = find_window_handle ("Labanator Special", handle)) >= 0) {
   active_window (x); /* Activate LABANATOR special */
   return -1;
  }

 /* Call the Dialog Routine */
  simple_dialog_open (handle, -5000, -5000, FALSE, info, data);
  return -1;
 }


/* LABAN project close */
int laban_proj_close (int handle, void *value) {
  return -1;
 }

/* Find location on the stave with mouse cursor */
bool find_stave_loc (int handle, LABAN_STRUCT *labandata, int *sx, int *sy, int *row, int *x, int *pya, int *pyb) {
 int c = 0, i, ii, vs, vr, px, py;

 *sx = *sy = -1;
 if (win_get_mouse(handle, &px, &py)) return TRUE; /* Can the coords be gotten? */
 vs = labandata->labanator.vertsize;
 vr = labandata->labanator.vertspc;
 if (px < labandata->labanator.farleft || px > labandata->labanator.farrght) return 0; /* Does it lie on a square of stave */
  /* Find the location on the stave */
   for (c = 0; c < 16; c++) {
    if (px > labandata->labanator.stavex[c] && px < labandata->labanator.stavex[c+1]) {
     *sx = c;
     *x = (labandata->labanator.stavex[c] + labandata->labanator.stavex[c + 1]) >> 1;
     break;
    }
   }
  if (*sx < 0) return TRUE; /* Not within a square of the stave */
 /* Now find the Y position */
   for (c = 0; c < vr; c++) {
    if (labandata->options.read_dir) {
     *pya = (vs-1) * c / vr;
     *pyb = (vs-1) * (c + 1) / vr;
     if (py > *pya && py < *pyb) { *sy = c; break; }
    } else {
     *pya = ((vs-1) * (vr - c - 1) / vr);
     *pyb = ((vs-1) * (vr - c) / vr);
     if (py > *pya && py < *pyb) { *sy = c; break; }
    }
   }
  *row = labandata->scroll + *sy;

  return FALSE;
 }

/* The polygon glyph drag resize routine */
int laban_drag_polygon_resize (int handle, void *value) {
  return 0;
 }

/* The polygon glyph drop resize routine */
int laban_drop_polygon_resize (int handle, void *value) {
  LABAN_STRUCT *labandata = (LABAN_STRUCT *) value;
  bool refresh = FALSE;
  int sx, sy, row, x, pya, pyb, i, ii, ps, pss = -1,
      lrow = labandata->labanator.row;

  if (win_dragend (handle)) return -1;
  if (find_stave_loc (handle, labandata, &sx, &sy, &row, &x, &pya, &pyb)) return 0;
  if (sx != labandata->labanator.sx) return 0; /* Cursor in different column */
  if (sy < (ps = labandata->labanator.sy)) return 0; /* Can not resize */
  if (row > (lrow + 0xff)) return 0; /* Hardly likely, but a precaution */
   if (row >= labandata->maxpanto) {
    if (laban_proj_stave_extend (labandata, row + 1)) return 0; /* Extend stave */
    refresh = TRUE; /* Redraw all the stave */
   }
   if (row < labandata->maxpanto) {
    ii = lrow + 1;
      while (ii <= row) {
       if (labandata->panto[ii].stave.symbol[sx] < 0xfe) goto do_refresh;
       ii++;
      }
     ii = lrow + labandata->panto[lrow].stave.length[sx];
     pss = ps + labandata->panto[lrow].stave.length[sx];
     /* Destroy the old length of glyph */
      while (ii > lrow) {
       labandata->panto[ii--].stave.symbol[sx] = 0xff;
      }
     i = labandata->panto[lrow].stave.length[sx] = row - lrow;
     ii = lrow + 1;
      while (i) {
       labandata->panto[ii].stave.symbol[sx] = 0xfe;
       labandata->panto[ii].stave.length[sx] = ii - lrow - 1;
       ii++; i--; }
     }
    do_refresh:
     if (!refresh) {
      if (pss != -1) {
       if (sy > pss) { pss = sy; }
       laban_proj_drawer (NULL, handle, labandata, ps, pss + 1, labandata->scroll + ps);
      }
     } else
      laban_proj_drawer (NULL, handle, labandata, 0, labandata->labanator.vertspc, labandata->scroll);

  return 0;
 }


/* The End of the Drag for the Laban project (mode one) */
int laban_proj_drop_mde_one (int handle, void *value) {
 LABAN_STRUCT *labandata = (LABAN_STRUCT *) value;
 POPUP_PALETTE *pal = labandata->labanator.special;
 int sym;

 /* Polyglyph */
  int mde_one_polyglyph () {
   int ii, cc, ps;
   bool refresh = FALSE;

   ps = labandata->labanator.sy;
   cc = labandata->labanator.sx;
   ii = labandata->labanator.row;
    if (ii >= labandata->maxpanto) {
     if (laban_proj_stave_extend (labandata, ii + 1)) return 0; /* Extend stave */
     refresh = TRUE; /* Redraw all the stave */
    }
    if (ii < labandata->maxpanto) {
     labandata->panto[ii].stave.symbol[cc] = sym;
     labandata->panto[ii].stave.shader[cc] = labandata->labanator.polygons;
     labandata->panto[ii].stave.length[cc] = 0x00;
     labandata->panto[ii].stave.modifier[cc] = 0x00;
     labandata->panto[ii].stave.align[cc] = 120;
    }
    if (!refresh)
     laban_proj_drawer (NULL, handle, labandata, ps, ps + 1, labandata->scroll + ps);
    else
     laban_proj_drawer (NULL, handle, labandata, 0, labandata->labanator.vertspc, labandata->scroll);
    return 0;
   }

 /* Space Glyph */
  int mde_one_ss () {
   int ii, cc, ps;
   bool refresh = FALSE;

   ps = labandata->labanator.sy;
   cc = labandata->labanator.sx;
   ii = labandata->labanator.row;
    if (ii >= labandata->maxpanto) {
     if (laban_proj_stave_extend (labandata, ii + 1)) return 0; /* Extend stave */
     refresh = TRUE; /* Redraw all the stave */
    }
    if (ii < labandata->maxpanto) {
     labandata->panto[ii].stave.symbol[cc] = 0x30;
     labandata->panto[ii].stave.shader[cc] = sym;
     labandata->panto[ii].stave.modifier[cc] = 0x00; /* Specify no modifier */
    }
    if (!refresh)
     laban_proj_drawer (NULL, handle, labandata, ps, ps + 1, labandata->scroll + ps);
    else
     laban_proj_drawer (NULL, handle, labandata, 0, labandata->labanator.vertspc, labandata->scroll);
    return 0;
   }

  destroy_overlay (labandata->labanator.popup);
  labandata->labanator.popup = -1; /* Signal the end of pop-up */
  if (win_dragend (handle)) return -1;

  sym = pal[labandata->labanator.popup_stx + 1];
   if (labandata->labanator.popup_stx >= 0) {
    switch (pal[labandata->labanator.popup_stx]) {
      case POPUP_POLYGON: { /* Check for symbol type */
       return mde_one_polyglyph (); }
      case POPUP_SS: {
       return mde_one_ss (); }
     }
    }

  return 0;
 }

/* LABAN Polyglyph state change */
int laban_proj_drop_mde_two (int handle, void *value) {
 LABAN_STRUCT *labandata = (LABAN_STRUCT *) value;
 POPUP_PALETTE *pal = labandata->labanator.special;
 UBYTE shader;
 int sym, ii, cc, ps, ln;

  destroy_overlay (labandata->labanator.popup);
  labandata->labanator.popup = -1; /* Signal the end of pop-up */
  if (win_dragend (handle)) return -1;

   if (labandata->labanator.popup_stx >= 0) {
    sym = pal[labandata->labanator.popup_stx + 1];
    ps = labandata->labanator.sy;
    cc = labandata->labanator.sx;
    ii = labandata->labanator.row;
    /* Find out about the symbol */
     switch (pal[labandata->labanator.popup_stx]) {
       case POPUP_SHADE: {
         ln = labandata->panto[ii].stave.length[cc] + 1;
         labandata->labanator.polygons = sym; /* Set new polyshade default */
         labandata->panto[ii].stave.shader[cc] = sym; break;
        }
       case POPUP_SS_C: { /* Change space glyph */
         ln = 1; shader = labandata->panto[ii].stave.shader[cc];
         labandata->panto[ii].stave.shader[cc] = (shader & 0x7c) | (sym & 0x83);
         break;
        }
      }
    laban_proj_drawer (NULL, handle, labandata, ps, ps + ln, labandata->scroll + ps);
   }

  return 0;
 }

/* LABAN project mouse interact */
int laban_proj_interact (int handle, void *value) {
 LABAN_STRUCT *labandata = (LABAN_STRUCT *) value;
 BITMAP *bitmap;
 unsigned char mde;
 int popup, x, y, pya, pyb, i, ii, glyph,
     sx = -1, sy = -1, /* The position on the stave */
     px, py, vs, vr, row, shade,
     c = 0; /* Counter */

  bool polyglyph_palette () { /* The polygon palette */
  /* Can the bitmap be created */
   if ((popup = popup_palette_draw (polyglyph_popup, handle,
               labandata->labanator.polygons, 0, x, pya, pyb, labandata,
               laban_proj_drop_mde_one)) < 0)
               return FALSE;
   return FALSE;
  }

  bool ss_palette () { /* The spacing symbol palette */
   if ((popup = popup_palette_draw (ss_popup, handle, 0, 0, x, pya, pyb,
               labandata, laban_proj_drop_mde_one)) < 0)
               return FALSE;
   return FALSE;
  }

  bool polygon_drag_resize () {
   labandata->labanator.popup_stx = sx;
   labandata->labanator.popup_sty = sy;
   win_dragstart (handle, laban_drag_polygon_resize, laban_drop_polygon_resize);
   return FALSE;
  }

 /* Change the shade of the polygon glyph, or add pins to it */
  bool polyglyph_change_state () {
  /* Can the bitmap be created */
   if ((popup = popup_palette_draw (polyshade_popup, handle,
               labandata->labanator.polygons, 0, x, pya, pyb, labandata,
               laban_proj_drop_mde_two)) < 0)
               return FALSE;
   return FALSE;
  }

 /* Setup the Space Glyph Change Palette */
  bool ss_change_state () {
  /* Can the bitmap be created */
   if ((popup = popup_palette_draw (ss_popup_c, handle, shade, 0,
               x, pya, pyb, labandata, laban_proj_drop_mde_two)) < 0)
               return FALSE;
   return FALSE;
  }

 /* Toggle between double lined, and single lined */
  bool ss_toggle_state () {
   int shader = labandata->panto[ii].stave.shader[sx];

   labandata->panto[ii].stave.shader[sx] ^= 0x80;
   laban_proj_drawer (NULL, handle, labandata, sy, sy + 1, labandata->scroll + sy);
   return FALSE;
  }

 /* Make sure that only one mouse button is pressed */
  if (mouse_status == 0x01) /* Left Mouse Button */
   mde = labandata->labanator.leftmouse;
  else if (mouse_status == 0x02) /* Right Mouse Button */
   mde = labandata->labanator.rightmouse;
  else if (mouse_status == 0x04) /* Middle Mouse Button */
   mde = labandata->labanator.midmouse;
  else /* Too many mouse buttons are being pressed */
   return 0;

 /* Can the drag controls be initialized? */
 if (labandata->labanator.popup >=0 || win_dragavail ()) return 0;
 if (find_stave_loc (handle, labandata, &sx, &sy, &row, &x, &pya, &pyb)) return 0;

  /* Enter Mode */
  labandata->labanator.row = ii = row;
  labandata->labanator.sx = sx; labandata->labanator.sy = sy;
  if (sy < 0) return 0; /* Could not find the row */

   if (ii >= labandata->maxpanto || (glyph = labandata->panto[ii].stave.symbol[sx]) == 0xff) { /* No Glyph */
    switch (mde) {
     case 0x1: { /* Place Polyglyph */
       if (ii) { /* if within the grid */
        return (polyglyph_palette () ? -1 : 0); /* The polyglyph_palette */
       }
      break; }
     case 0x2: { /* Place an one-square glyph */
       if (ii) { /* if within the grid */
        return (ss_palette () ? -1 : 0 ); /* The spacing glyph palette */
       }
      break; }
    }
   } else if (ii > 0) { /* Before the permittable area glyph */
    shade = labandata->panto[ii].stave.shader[sx];
    if (glyph < 0x30) { /* Beginning of glyph */
      if (mouse_status == 0x01) {
       return (polygon_drag_resize () ? -1 : 0); /* Resize polyglyph */
      } else if (mouse_status == 0x02) {
       if (glyph < 0x28) {
        return (polyglyph_change_state () ? -1 : 0); /* Change state of polyglyph */
       } else { /* Reversed for vertical lines */
       }
      }
     } else if (glyph == 0x30) { /* 0x30: Space Glyph */
      if (mouse_status == 0x01) { /* Left Mouse Button */
       return (ss_toggle_state () ? -1: 0); /* Toggle between states of space glyph */
      } else if (mouse_status == 0x02) { /* Right Mouse Button */
       return (ss_change_state () ? -1: 0); } /* Change State of Space Glyph */
     } else if (glyph == 0xfe) { /* Midpoint of a polygon glyph */
      if (ii < labandata->maxpanto) {
       do { ii--; sy--; } while (labandata->panto[ii].stave.symbol[sx] == 0xfe);
       labandata->labanator.row = ii; labandata->labanator.sy = sy;
        if (mouse_status == 0x01) {
         return (polygon_drag_resize () ? -1 : 0); /* Resize polyglyph */
        } else if (mouse_status == 0x02) {
         return (polyglyph_change_state () ? -1 : 0); /* Change state of polyglyph */
        }
     } /* if (glyph < 0x30) ... */
    } /* labandata->maxpanto */
   } /* (glyph == ... */
  return 0;
 }

/* LABAN project thread */
int laban_proj_thread (int handle, void *value) {
  return 0;
 }

/* LABAN project activate */
int laban_proj_active (int handle, void *value) {
  win_multithread(handle, laban_proj_thread, LABAN_THREAD_RATE);
  return 0;
 }

/* LABAN project inactivate */
int laban_proj_inactive (int handle, void *value) {
  win_multithread(handle, NULL, 0); /* No threading is permitted */
  return 0;
 }

/* LABAN project redraw */
int laban_proj_redraw (int handle, void *value) {
  LABAN_STRUCT *labandata = (LABAN_STRUCT *) value;
  BITMAP *bitmap;
  int c = 0, x = 0, y = 0, i = 0, yy, y_ , ii,
      vr, /* Vertical rows */
      vs, /* Vertical size */
      fl, /* Far left of the stave */
      fr, /* Far right of the stave */
      mos, /* Middle of stave */
      sl, sl_ii, /* Left Stave Line */
      sm, sm_ii, /* Middle Stave Line */
      sr, sr_ii, /* Right Stave Line */
      winh, winv;

#ifdef DEBUG
 fprintf(stderr, "labanscr.c: laban_proj_redraw is being executed at: %u\n", gui_timer);
#endif

  if (draw_window (handle)) return -1; /* Draw the window */
  bitmap = window_bitmap(handle);

  winh = window_h(handle);
  winv = window_v(handle);
  vs = labandata->labanator.vertsize= winv;
   switch (labandata->parts == 1) {
    case 1: { /* Step for one score */
     vr = labandata->labanator.vertspc = winv * 20 / winh; /* Vertical rows */
     fl = labandata->labanator.farleft =    2 * (winh-1) / 20;
     fr = labandata->labanator.farrght =   18 * (winh-1) / 20;
     sl = labandata->labanator.leftline =   5 * (fr-fl) / 16 + fl;
     sm = labandata->labanator.midline =    8 * (fr-fl) / 16 + fl;
     sr = labandata->labanator.rightline = 11 * (fr-fl) / 16 + fl;
     break; }
    case 2: { /* Step for two score */
     vr = labandata->labanator.vertspc = winv * 36 / winh; /* Vertical rows */
     fl = labandata->labanator.farleft =          2 * (winh-1) / 36;
     fr = labandata->labanator.farrght =         34 * (winh-1) / 36;
     sl = labandata->labanator.leftline =         5 * (fr-fl) / 32 + fl;
     sm = labandata->labanator.midline =          8 * (fr-fl) / 32 + fl;
     sr = labandata->labanator.rightline =       11 * (fr-fl) / 32 + fl;
     mos = labandata->labanator.stave_mid =      16 * (fr-fl) / 32 + fl;
     sl_ii = labandata->labanator.leftline_ii =  21 * (fr-fl) / 32 + fl;
     sm_ii = labandata->labanator.midline_ii =   24 * (fr-fl) / 32 + fl;
     sr_ii = labandata->labanator.rightline_ii = 27 * (fr-fl) / 32 + fl;
     break; }
    default: { exit(255); /* Error! */ }
   }

  /* Calculate the positions in the stave */
   for (c = 0; c < 1 + (labandata->parts * labandata->width); c++) {
    labandata->labanator.stavex[c] = (c * (fr-fl) / 16) + fl; }

  /* Draw the stave */
  laban_proj_drawer (bitmap, -1, labandata, 0, vr, labandata->scroll);
  blit_window (handle); /* Also, unhides mouse! */
  return 0;
 }

/* Convert Device Indepedent Colours to Graphic Mode Specific Colours */
void get_hware_specific_colors (LABAN_STRUCT *data) {
 int i;

   for (i = 0; i < maxlabancolors; i++) {
     data->labanator.hs_colors[i] = makecol (
     data->options.colors[i].r,
     data->options.colors[i].g,
     data->options.colors[i].b);
    }

 }

/* Start a new laban project */
int laban_new_project (int handle, DIALOG_INFO *info, DIALOG_DATA *data) {
  int newhandle, c, cx, cy, sx, sy;
  bool error = FALSE;
  LABAN_STRUCT *labandata = malloc(sizeof (LABAN_STRUCT));
  char buffer [256];

   if (labandata == NULL) { /* Unable to allocate data to store LABAN information */
    return -1;
   }

  labandata->saved = TRUE;
  labandata->newproject = TRUE;
 /* Choose a "default" project depending on the environment */
  sprintf(buffer, (_use_lfn ? "pantomime%u" laban_ext_lfn : "pant%u" laban_ext) , laban_count);
  if ((labandata->filename = strdup (buffer)) == NULL) { free(labandata); return -1; } /* Copy string to array */

 /* Initialize the Attribution Details */
  labandata->attribution.title = strdup ("");
  labandata->attribution.bard = strdup ("");
  labandata->attribution.scriber = strdup ("");
  labandata->attribution.type = strdup ("");
  labandata->attribution.chapter = strdup ("");
  labandata->attribution.movement = strdup ("");
  labandata->attribution.part = strdup ("");
  labandata->attribution.description = strdup ("");
  labandata->attribution.copyright = strdup ("");
  labandata->options = laban_option_defaults;
 /* Initialize the hardware specific colour table for labanator */
  get_hware_specific_colors (labandata);
 /* Initialize other LABANATOR specifics */
  labandata->labanator.popup = -1;
  labandata->labanator.polygons = 1; /* The default polygon dot mode */
 /* The mouse modes */
  labandata->labanator.leftmouse = 1;
  labandata->labanator.midmouse = 3;
  labandata->labanator.rightmouse = 2;
 /* Number of parts to work with */
  labandata->parts = 1;
  labandata->width = 16; /* Sixteen Column Stave */
  /* Start the first bit of pantomime data */
   cx = labandata->parts * labandata->width; /* Get total size */
   labandata->maxpanto = 5;
   labandata->scroll = 0;
   labandata->panto = malloc (sizeof (LABAN_PANTOMIME));
  /* Setup STAVE information */
   for (c = 0; c < 5; c++) {
    labandata->panto[c].stave.symbol   = malloc(sizeof(LABAN_PANTOMIME_SYMBOL)*cx);
   /* 0xff all the fields */
    memset(labandata->panto[c].stave.symbol, 0xff, sizeof(LABAN_PANTOMIME_SYMBOL)*cx);
    labandata->panto[c].stave.length   =  malloc(sizeof(LABAN_PANTOMIME_LENGTH)*cx);
    labandata->panto[c].stave.shader   =  malloc(sizeof(LABAN_PANTOMIME_SHADER)*cx);
    labandata->panto[c].stave.modifier =  malloc(sizeof(LABAN_PANTOMIME_MODIFY)*cx);
    labandata->panto[c].stave.align    =  malloc(sizeof(LABAN_PANTOMIME_ALIGN)*cx);
   }
  /* BAR information */
   labandata->panto[0].bar.type = 0;
   labandata->panto[0].bar.beats = 0x04;
   labandata->panto[0].bar.size = 0x81;
   labandata->panto[0].bar.tick = 0x01;
    for (c = 1; c < 4; c++) {
     labandata->panto[c].bar.type = 0xff;
     labandata->panto[c].bar.size = c - 1;
    }
   labandata->panto[4].bar.type = 1;
   labandata->panto[4].bar.beats = 3;
   labandata->panto[4].bar.size = labandata->panto[4].bar.tick = 4;
 /* Place a window in a somesort of cassading pattern */
   get_window_size (WIN_WITHFILE, 192, 400, &sx, &sy);
   c = (scrn_hght - sy) / 12; c = laban_count % (c+1);
   cx = c * 24; cy = c * 12;

 /* Create a window (link to its parent -- Rudolf von Laban Menu) */
   if ((newhandle =
        allocate_window (cx, cy, 240, 400, handle, /* set x to 240 */
          WF_SPECIAL, WIN_WITHFILE, "Labanator", labandata->filename,
          laban_proj_redraw,
          laban_proj_close,
          laban_proj_special,
          labandata)) >=0 ) {
    win_interact (newhandle, laban_proj_interact); /* Window Interaction Applet */
    win_activities (newhandle, laban_proj_active, laban_proj_inactive);
    win_multithread(newhandle, laban_proj_thread, LABAN_THREAD_RATE);
    active_window (newhandle);
    laban_count++;
    if (laban_count == 10000) laban_count = 0; /* Reset to zero */
   } else {
    /* Free row data */
     for (c = 0; c < 5; c++) {
      free(labandata->panto[c].stave.symbol);
      free(labandata->panto[c].stave.length);
      free(labandata->panto[c].stave.shader);
      free(labandata->panto[c].stave.modifier);
     }
    free (labandata); /* Free LABAN data struct */
   }
  return -1;
 }

/*
  LABAN code legend:
   (The bar line appears after the row, no glyphs appear on row zero)

  To maintain forward-compatibility: all reserved values must be set to
  zero, unless stated in the specifications.

  LABAN_PANTOMIME_STAVE.symbol
    0x00: Spacer/Crater (to align special modifiers in centre)
    0x01 - 0x18: Polygon Symbols. (see: "symbols.c")
    0x19 - 0x27: Reserved for future polygon symbols.
    0x28 - 0x2f: Reserved for vertical line glyphs.
    0x30:        Space Glyph (Partly implemented)
    0x31 - 0xe0: Reserved
    0xe0-0xef: Reserved
    0xf0-0xfc: Reserved
    0xfd: Reserved for symbol on the vertical line glyph.
    0xfe: Continuation of a glyph
    0xff: No glyph

  LABAN_PANTOMIME_STAVE.length
    (length of a symbol)
    (on .type 0xfe, specifies the count back for symbol information)

  LABAN_PANTOMIME_STAVE.shader
    [Polygon. Symbol == 0x01 - 0x27 ]
     (Shade type of the symbol)
           On a polygon
     value & 0x07 = (Shading type)
      0x00: No shading
      0x01: Dot
      0x02: Cross
      0x03: Two side lines (Not Implemented)
      0x04: Shade
      0x05: Filled
      0x06 - 0x0f: Reserved
     value & 0xf8 = (Reserved for future)
    [Space. Symbol == 0x30: Space Glyph]
     value & 0x01 = (Top Pin,    location varies on rotation)
     value & 0x02 = (Bottom Pin, location varies on rotation)
     value & 0x7c = (Glyph ID -- see symdata (value >> 2) == glyph ID )

  LABAN_PANTOMIME_STAVE.modifier
    [Polygon. Symbol == 0x01 - 0x27 not implemented ]
     (The pin on the Symbol)
     value & 0x03 = (Pin Type)
      0x00 = Square Filled
      0x01 = Square Outlined
      0x02 = Circle Filled
      0x03 = Circle Outlined
      0x04 = Nail Ended
      0x05 = Double Nail
      0x06 - 0x07 = reversed
     value & 0x08 = (Reversed)
     value & 0xf0 = (Pin direction)
      0x00 = pin is non-existant
      0x10 = North
      0x20 = North - East
      0x30 = East
      0x40 = South - East
      0x50 = South
      0x60 = South - West
      0x70 = West
      0x80 = North - West
      0x90 = Dual ended west to east
      0xa0 = Dual ended north east to south west
      0xb0 = Dual ended north to south
      0Xc0 = Dual ended north west to south east
      0xd0 - 0xf0 = Reserved
    [Space. Symbol == 0x30 (partly implemented)]
     value & 0x03 = anti-clock wise rotation in steps of 90-degrees
      0 = zero degrees
      0x40 = 90 degrees left
      0x80 = 180 degrees
      0xC0 = 90 degrees right
     value & 0xfc = reseved, must be set to zero

  LABAN_PANTOMIME_BAR.type
    0x00: Single Line
    0x01: Double Line
   (0x02 - 0xdf) reserved for the future
    0xef: No Line, but inside an untimed bar.
   (0xe0 - 0xee) reserved for untimed marks.
    0xf0: Tick Mark (Implemented)
    0xff: No Line, but timed (Just a referal number)
   (0xf1 - 0xfe) reserved for timed marks.
  LABAN_PANTOMIME_BAR.beats (& 0x3f)
    (Specifies the number of musical ticks per bar:
     the numerator in music notation time signature)
    0x00: The bar that appears after it, is untimed.
  LABAN_PANTOMIME_BAR.tick  (& 0x3f)
    (Specifies the musical tick size:
     the demominator of music notation time signature)
  LABAN_PANTOMIME_BAR.size  (& 0x3f)
    (Number of rows for each musical tick)
    (When 0x80 bit is set, no ticks marks are permitted)
    (on .type 0xf0 or higher, number of rows to go back to get bar info)
 */

