/*
 * Header File for LABANscript Applet Routines (Header File)
 *
 * Written and Copyright 1999 by David E. Down the Faery Monarch, Gonksoftware
 * This software is released under the GNU GPL version 2.
 *
 * See the file "COPYING" for software license details.
 */

#ifndef __GSW_LABANSCR_H__
#define __GSW_LABANSCR_H__

#ifndef bool
#define bool char
#endif

#define laban_ext     ".rvl"  /* The extension for Rudolf von Laban files */
#define laban_ext_lfn ".rvlf" /* The long file name extension */

/* LABAN colour table */
#define maxlabancolors 9
#define hs_labangrid  labanator.hs_colors[0] /* Grid Lines */
#define hs_labanstave labanator.hs_colors[1] /* Stave Lines */
#define hs_labandivid labanator.hs_colors[2] /* Part Divider */
#define hs_labanbkgrd labanator.hs_colors[3] /* Background */
#define hs_labancrate labanator.hs_colors[4] /* Pin crater */
#define hs_labanplyol labanator.hs_colors[5] /* Polygon outline */
#define hs_labanpysnd labanator.hs_colors[6] /* Polygon secondary */
#define hs_labanpymrk labanator.hs_colors[7] /* Polygon mark */
#define hs_labanspace labanator.hs_colors[8] /* Spacing symbols */

/* LABAN attribution structure */
 typedef struct {
  char *title;      /* Pantomime Title */
  char *bard;       /* Body Animation Routine Designer */
  char *scriber;    /* Notation Scriber */
  char *type;       /* Pantomime Type */
  char *chapter;    /* Chapter */
  char *movement;   /* Movement */
  char *part;       /* Part */
  char *description;/* Description */
  char *copyright;  /* Copyright */
 } LABAN_ATTRIBUTION;

/* LABAN options */
 typedef struct {
  bool read_dir;  /* Reading Direction (TRUE for top-to-bottom) */
  bool grid_mode; /* The grid mode */
  bool use_crater;/* Use crater */
  char dialect;   /* Laban Dialect */
  RGB  colors [maxlabancolors]; /* Hardware Independent Colours */
 } LABAN_OPTIONS;

/* For LABANATOR use only */
 typedef struct {
  int vertspc;                    /* Number of Vertical Rows */
  int vertsize;                   /* Area for Vertical Data */
  int farleft,                    /* Far left of the stave */
      leftline, leftline_ii,      /* Left LABAN Stave Line */
      midline, midline_ii,        /* Middle LABAN Stave Line */
      rightline, rightline_ii,    /* Right LABAN Stave Line */
      stave_mid,                  /* Stave Middle (Only for Step for Two) */
      farrght,                    /* Far right of the stave */
      polygons,                   /* Default polygon shade */
      sx, sy, row,                /* Selection location */
      popup,                      /* Popup Window ID */
      popup_stx, popup_sty;       /* Popup section status */
  void *special;                  /* For special applet use only */
  int stavex [49];                /* The coords for all the stave x-lines */
  int hs_colors [maxlabancolors]; /* Hardware specific colours */
  unsigned char leftmouse, midmouse, rightmouse; /* The Mouse Button modes */
 } LABANATOR_USE;

/* Pantomime Data Project (Allocate As An Array) */
#define LABAN_PANTOMIME_SYMBOL unsigned char
#define LABAN_PANTOMIME_LENGTH unsigned char
#define LABAN_PANTOMIME_SHADER unsigned char
#define LABAN_PANTOMIME_MODIFY unsigned char
#define LABAN_PANTOMIME_ALIGN  unsigned char
 typedef struct {
   unsigned char type,  /* Bar type */
                 beats, /* Ticks per bar (Numerator in Music Time Signature) */
                 tick,  /* Tick size (Demoninator in Music Time Signature) */
                 size;  /* Size space for each beat */
  } LABAN_PANTOMIME_BAR;
 typedef struct {  /* (16/24 for one, 32/48 for two) */
   LABAN_PANTOMIME_SYMBOL *symbol;   /* Symbol */
   LABAN_PANTOMIME_LENGTH *length;   /* Length */
   LABAN_PANTOMIME_SHADER *shader;   /* Shader */
   LABAN_PANTOMIME_MODIFY *modifier; /* Modifier */
   LABAN_PANTOMIME_ALIGN  *align;    /* Modifier Aligment, center is 120 */
  } LABAN_PANTOMIME_STAVE;
 typedef struct {
  LABAN_PANTOMIME_BAR    bar;
  LABAN_PANTOMIME_STAVE  stave;
 } LABAN_PANTOMIME;

/* Main LABAN structure */
 typedef struct {
  bool saved;                     /* Has the project been saved */
  bool newproject;                /* This is a new project */
  char *filename;                 /* The Project's Filename */
  unsigned char parts;            /* Number of staves in one project */
  unsigned char width;            /* The width of the stave */
  LABANATOR_USE labanator;        /* For Labanator Use Only Parameters */
  LABAN_ATTRIBUTION attribution;  /* Attribution Info */
  int maxpanto, scroll;           /* Amount of Pantomimic Data */
  LABAN_PANTOMIME *panto;         /* Pantomimic Data */
  LABAN_OPTIONS options;          /* LABANATOR options */
 } LABAN_STRUCT;

int laban_new_project (int handle, DIALOG_INFO *info, DIALOG_DATA *data); /* Start new LABAN project */
void laban_attributive_info (int parent, LABAN_STRUCT *value); /* LABAN attributive information box */
int laban_attributive_dialog (int handle, DIALOG_INFO *info, DIALOG_DATA *data); /* Caller to LABAN attributive information box */

extern char *labancolors[];
extern RGB  rgblabancolors[];
extern int  smflabancolors[];

#endif

