/*
 * The MAIN program of the MABANOL program
 *
 * This program is a LABAN based kinetographic editor. Kinetography is the
 * art of noting down body animation.
 *
 * ===========================================================================
 *
 * Written and Copyright 1999 by David E. Down the Faery Monarch, Gonksoftware
 * This software is released under the GNU GPL version 2.
 *
 * See the file "COPYING" for software license details.
 */


#include <string.h>
#include <time.h>
#include "def.h"      /* Defines */
#include "hslpal__.h" /* Include the palette */
#include "gui.h"
#include "labanscr.h"
#include "mabanol.h"

/* There are sixteen columns to take care of in the
   Lousy Alien Body Animation Notation that was proposed by Rudolf von Laban

 : 0 : 1 : 2 : 3 : 4 | 5 : 6 : 7 | 8 : 9 : A | B : C : D : E : F :
 :   :   :   :   :   |   :   :   |   :   :   |   :   :   :   :   :
 :   :   :   :   :   |   :   :   |   :   :   |   :   :   :   :   :
 :   :   :   :   :   |___:___:___|___:___:___|   :   :   :   :   :

   But I have extended the specifications to allow for 24 Columns, which I
   have not implemented yet!

 */

/* Graphic Patterns for drawing items on the LABAN stave */
BITMAP *laban_dline, *laban_hash;

/* Open up the SDK window (closing down ABOUT) */
#ifdef DJGPP
int call_up_SDK_window (int handle, DIALOG_INFO *info, DIALOG_DATA *data) {

 DIALOG_INFO new_info [] = {
    {"Software Development Kit credits", 352, 112, WF_NO_OPTIONS, WIN_STANDARD, NULL}
   };
 DIALOG_DATA new_data [] = {
    {8, 8,  1, 0, 0, 0, clrsh_winfore, "This software contains software that was", biosfont, NULL, 0},
    {8, 16, 1, 0, 0, 0, clrsh_winfore, "developed by University of California,", biosfont, NULL, 0},
    {8, 24, 1, 0, 0, 0, clrsh_winfore, "Berkeley and its contributors.", biosfont, NULL, 0},
    {8, 40, 1, 0, 0, 0, clrsh_winfore, "This software uses DJGPP development tools" ,biosfont, NULL, 0},
    {24, 48,1, 0, 0, 0, clrsh_winfore, "http://www.delorie.com", biosfont, NULL, 0},
    {8, 56, 1, 0, 0, 0, clrsh_winfore, "This software uses Allegro libraries",biosfont, NULL, 0},
    {24, 64,1, 0, 0, 0, clrsh_winfore, "http://www.tabula.demon.co.uk/allegro", biosfont, NULL, 0},
    {8, 88, 1, 0, 0, 0, clrsh_winfore, "The Faery Monarch is one of", biosfont, NULL, 0},
    {8, 96, 1, 0, 0, 0, clrsh_winfore, "David E. Down's pseudonyms", biosfont, NULL, 0},
    {296, 80, DIALOG_BUTTON, 48, 24, 0, 0, "Okay", NULL, simple_dialog_close, 'O'},
    {0, 0, 0, 0, 0, 0, 0, NULL, NULL, simple_dialog_close, 0}
   };

#ifdef DEBUG
 fprintf(stderr, "mabanol.c: call_up_SDK_window is being executed at: %u\n", gui_timer);
#endif

 /* Call the Dialog Routine */
  if (simple_dialog_open (-1, -5000, -5000, FALSE, new_info, new_data) < 0) return -1;
  free(info);
  free(data);
  return 0; /* Complete close is permitted */
 }
#endif

/* Test out the GUI */
int about_laban () {
 DIALOG_INFO info [] = {
    {"About L.A.B.A.N.", 360, 176, WF_NO_OPTIONS, WIN_STANDARD, NULL}
   };
 DIALOG_DATA data [] = {
    {180, 8, 2, 0, 0, 0, clrsh_winfore,"Lean Against Bricks And Nails", biosfont, NULL, 0},
    {180, 16,2, 0, 0, 0, clrsh_winfore,"LABAN based kinetographic editor", biosfont, NULL, 0},
    {180, 24,2, 0, 0, 0, clrsh_winfore,"A Gonksoftware Computer Program", biosfont, NULL, 0},
    {8, 40, 1, 0, 0, 0, clrsh_winfore, "Written By: David E Down" ,biosfont, NULL, 0},
    {8, 48, 1, 0, 0, 0, clrsh_winfore, "http://members.tripod.com/~faerymonarch/", biosfont, NULL, 0},
    {8, 56, 1, 0, 0, 0, clrsh_winfore, "mailto:faeries@mpx.com.au", biosfont, NULL, 0},
    {8, 64, 1, 0, 0, 0, clrsh_winfore, "Copyright 1999 David E Down, Gonksoftware", biosfont, NULL, 0},
    {8, 80, 1, 0, 0, 0, clrsh_winfore, "This program utilizes the Largely Advanced", biosfont, NULL, 0},
    {8, 88, 1, 0, 0, 0, clrsh_winfore, "Body Animation Notation language that was", biosfont, NULL, 0},
    {8, 96, 1, 0, 0, 0, clrsh_winfore, "proposed by Rudolf von Laban at the year", biosfont, NULL, 0},
    {8, 104, 1, 0, 0, 0, clrsh_winfore, "1928.", biosfont, NULL, 0},
    {8, 120,1, 0, 0, 0, clrsh_winfore, "Released under the GNU GPL version 2.", biosfont, NULL, 0},
    {8, 128,1, 0, 0, 0, clrsh_winfore, "Please see the file \"COPYING\" for details.", biosfont, NULL, 0},
    {8, 160,1, 0, 0, 0, clrsh_winfore, prog_version, biosfont, NULL, 0},
#ifdef DJGPP
    {216, 144, DIALOG_BUTTON, 80, 24, 0, 0, "SDK info", NULL, call_up_SDK_window, 'S'},
#endif
    {304, 144, DIALOG_BUTTON, 48, 24, 0, 0, "Okay", NULL, simple_dialog_close, 'O'},
    {0, 0, 0, 0, 0, 0, 0, NULL, NULL, simple_dialog_close, 0}
   };

#ifdef DEBUG
 fprintf(stderr, "mabanol.c: about_laban is being executed at: %u\n", gui_timer);
#endif

 /* Call the Dialog Routine */
  if (simple_dialog_open (-1, -5000, -5000, FALSE, info, data) < 0) return -1;
  return 0;
 }

/* Magic Button Menu selects L.A.B.A.N. */
int magic_button_laban (int handle, DIALOG_INFO *info, DIALOG_DATA *data) {
 int hx;

 /* Check if the about windows are open */
  if ((hx = find_window_handle ("About L.A.B.A.N.", -1)) >= 0) {
   active_window(hx);
   return -1; }
#ifdef DJGPP
  if ((hx = find_window_handle ("Software Development Kit credits", -1)) >= 0) {
   active_window(hx);
   return -1; }
#endif
 /* If not, open up an about window */
  about_laban ();
  return -1; /* Do not close */

 }

/* Close down Rudolf von Laban */
int rudolf_close (int handle, DIALOG_INFO *info, DIALOG_DATA *data) {
  int os;
  /* Check for an offspring */
   if ((os = window_offspring_of (handle)) >= 0) {
    return -1;
   } else {
    free (info);
    free (data);
    return 0;
   }
 }

/* Setup the Rudolf von Laban menu */
int setup_rudolf_menu () {
 int hx; /* Window Handle */

 DIALOG_INFO info [] = {
    {"Rudolf von Laban", 192, 96, WF_NO_OPTIONS, WIN_STANDARD, NULL}
   };
 DIALOG_DATA data [] = {
    {0, 0, DIALOG_BUTTON,  192, 24, 0, 0, "Start a new pantomime", NULL, laban_new_project, 'S'},
    {0, 24, DIALOG_BUTTON, 192, 24, 0, 0, "Open a pantomime file", NULL, NULL, 'O'},
    {0, 48, DIALOG_BUTTON, 192, 24, 0, 0, "Customize LABAN style", NULL, NULL, 'C'},
    {0, 72, DIALOG_BUTTON, 192, 24, 0, 0, "LABAN help", NULL, NULL, 'L'},
    {0, 0, 0, 0, 0, 0, 0, NULL, NULL, rudolf_close, 0}
   };

#ifdef DEBUG
 fprintf(stderr, "mabanol.c: setup_rudolf_menu is being executed at: %u\n", gui_timer);
#endif

 /* Call the Dialog Routine */
  if (hx = simple_dialog_open (-1, -5000, 48, FALSE, info, data) < 0) return -1;
 /* Open up the Laban Glyph Palette */

  return 0;
 }

/* Open up the Rudolf von Laban menu */
int magic_button_rudolf (int handle, DIALOG_INFO *info, DIALOG_DATA *data) {
  int x;
   if ((x = find_window_handle ("Rudolf von Laban", -1)) >= 0) {
    active_window(x);
    return -1; }
  setup_rudolf_menu ();
  return -1; /* Do not close */
 }

/* Unable to close applets on [Okay] */
int error_other_applets_end (int handle, DIALOG_INFO *info, DIALOG_DATA *data) {
  free (info);
  free (data);
  change_window_locks (FALSE, window_parent(handle), -1);
  return 0;
 }

/* Can Not Quit (Other applets are running) */
void error_other_applets (handle) {
 DIALOG_INFO info [] = {
    {"Unable to Close Down", 248, 72, WF_NOMINIMIZE, WIN_STANDARD, NULL}
   };
 DIALOG_DATA data [] = {
    {8, 8, 1, 0, 0, 0, clrsh_winfore,    "Please close down all applets", biosfont, NULL, 0},
    {8, 16,1, 192, 24, 0, clrsh_winfore, "before quitting Lean Against", biosfont, NULL, 0},
    {8, 24,1, 192, 24, 0, clrsh_winfore, "Bricks And Nails.", biosfont, NULL, 0},
    {192, 40, DIALOG_BUTTON,48, 24, 0, 0,"Okay", NULL, error_other_applets_end, 'O'},
    {0, 0, 0, 0, 0, 0, 0, NULL, NULL, error_other_applets_end, 0}
   };

   /* Call the Dialog Routine */
   simple_dialog_open (handle, -5000, -5000, TRUE, info, data);
 }

/* Quitting Lean Against Bricks And Nails */
int quit_laban_okay (int handle, DIALOG_INFO *info, DIALOG_DATA *data) {
  free(info);
  free(data);
  remove_mouse ();
  remove_timer ();
  set_gfx_mode (GFX_TEXT, 0, 0, 0, 0);
  printf("Exited from Lean Against Bricks And Nails! " prog_version "\n"
         "\n");
  exit(0);
 }

/* Can Not Quit (Other applets are running) */
void confirm_on_laban_quit (handle) {
 int num;
 DIALOG_INFO info [] = {
    {"Quitting!", 272, 88, WF_NOMINIMIZE, WIN_WITHFILE, NULL, "Lean Against Bricks And Nails"}
   };
 DIALOG_DATA data [] = {
    {8, 8, 1, 0, 0, 0, clrsh_winfore, NULL, biosfont, NULL, 0},
    {8, 16,1, 0, 0, 0, clrsh_winfore, NULL, biosfont, NULL, 0},
    {8, 24,1, 0, 0, 0, clrsh_winfore, NULL, biosfont, NULL, 0},
    {8, 40,1, 0, 0, 0, clrsh_winfore, "Do you want to quit L.A.B.A.N.?", biosfont, NULL, 0},
    {144, 56, DIALOG_BUTTON,64, 24, 0, 0,"Cancel",NULL, error_other_applets_end, 'C'},
    {216, 56, DIALOG_BUTTON,48, 24, 0, 0,"Okay",  NULL, quit_laban_okay, 'O'},
    {0, 0, 0, 0, 0, 0, 0, NULL, NULL, error_other_applets_end, 0}
   };

  /* Set up the proverbs? */
   switch ((num = random() % 4)) {
     case 0: {
      data[0].field = "Only real heroes kinetograph";
      data[1].field = "their heroic victory dances with";
      data[2].field = "Lean Against Bricks And Nails.";
      break; }
     case 1: {
      data[0].field = "Thank you for using Lean Against";
      data[1].field = "Bricks And Nails for your";
      data[2].field = "kinetographic requirements.";
      break; }
     case 2: {
      data[0].field = "May you not Lean Against Bricks";
      data[1].field = "And Nails while you perform your";
      data[2].field = "dance rituals.";
      break; }
     case 3: {
      data[0].field = "I hope that I have really helped";
      data[1].field = "you with kinetographing your";
      data[2].field = "latest pantomimes.";
      break; }
    }

  if (simple_dialog_open (handle, -5000, -5000, TRUE, info, data) < 0) return;
 }

/* The Magic Quit Button */
int magic_button_quit (int handle, DIALOG_INFO *info, DIALOG_DATA *data) {
   if (wininpos > 1) { /* Error Other Applets are Running */
    error_other_applets (handle);
   } else {
    confirm_on_laban_quit (handle);
   }
  return -1; /* Do not close */
 }

/* The Magic Menu Button */
int open_magic_button () {
  DIALOG_INFO info [] = {
    {"Launch Bar", 248, 24, WF_NOCLOSE | WF_NOMINIMIZE, WIN_NOBORDER, NULL}
   };
  DIALOG_DATA data [] = {
    {0,  0, DIALOG_BUTTON,56, 24, 0, 0,"About", NULL, magic_button_laban, 'A'},
    {56, 0, DIALOG_BUTTON,144,24, 0, 0,"Rudolf von Laban", NULL, magic_button_rudolf, 'R'},
    {200,0, DIALOG_BUTTON,48 ,24, 0, 0,"Quit", NULL, magic_button_quit, 'Q'},
    {0, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL, 0}
   };

 /* Call the Dialog Routine */
  if (simple_dialog_open (-1, 0, scrn_hght - 24, FALSE, info, data) < 0) return -1;
  return 0; /* Complete close is permitted */
 }

/* Selected graphic mode */
int graph_mode = 1;

/* Initialize the Program */
int initialize () {
 int mask;
  printf("Lean Against Bricks And Nails\n"
         "Body Animation Script Inscribed Code editor for the LABANscript language\n\n"
         "Initializing...\n");

  srandom(time (NULL));
   switch (clr_mode) {
    case 8: { mask = MASK_COLOR_8; break; }
    case 15:{ mask = MASK_COLOR_15; break; }
    case 16:{ mask = MASK_COLOR_16; break; }
   }
/* Setup the graphics mode */
  initialize_gui ();
  set_color_depth (clr_mode);
   switch (graph_mode) {
    case 3: {
      if (set_gfx_mode (GFX_AUTODETECT, 1024, 768, 1024, 768)) {
       printf("\nUnable to Initialize Graphics Mode.\n"); exit (255);
      } else {
       scrn_cntr = 512;
       scrn_widt = 1024;
       scrn_hght = 768;
      }
     break; }
    case 2: {
      if (set_gfx_mode (GFX_AUTODETECT, 800, 600, 800, 600)) {
       printf("\nUnable to Initialize Graphics Mode.\n"); exit (255);
      } else {
       scrn_cntr = 400;
       scrn_widt = 800;
       scrn_hght = 600;
      }
     break; }
    case 1: {
      if (set_gfx_mode (GFX_AUTODETECT, 640, 480, 640, 480)) {
       printf("\nUnable to Initialize Graphics Mode.\n"); exit(255);
      } else {
       scrn_cntr = 320;
       scrn_widt = 640;
       scrn_hght = 480;
      }
     break; }
   }

  make_rgb_map (hslpal_8grey);/* Set the palette */

/* Setup graphic patterns for LABANscript */
  laban_dline = create_bitmap(2, 2); clear_to_color (laban_dline, mask);
  putpixel (laban_dline, 0, 0, 1);
  putpixel (laban_dline, 1, 1, 1);
/* HASH */
  laban_hash = create_bitmap(4, 4); clear_to_color (laban_hash, mask);
  putpixel (laban_hash, 0, 0, 1);
  putpixel (laban_hash, 1, 1, 1);
  putpixel (laban_hash, 2, 2, 1);
  putpixel (laban_hash, 3, 3, 1);
 /* Get the wallpaper */
  wallpaper_setup ();
 /* Clear the screen */
  blit (wallpaper, screen, 0, 0, 0, 0, scrn_widt, scrn_hght);
  open_magic_button ();

 }

/* Interpret the command line parameters */
void phasecmdline (int noparams, char *params[]) {
  char *charpnt;
  int i;

   for (i = 1; i < noparams; i++) {
    charpnt = params[i];
     if (strcmp(charpnt, "-640") == 0) { /* 640x480 mode */
      graph_mode = 1;
     } else if (strcmp (charpnt, "-800") == 0) { /* 800x600 mode */
      graph_mode = 2;
     } else if (strcmp (charpnt, "-1024") == 0) { /* 1024x768 mode */
      graph_mode = 3;
     } else if (strcmp (charpnt, "-16") == 0) { /* 16-bit colour */
      clr_mode = 16;
     } else if (strcmp (charpnt, "-15") == 0) { /* 15-bit colour */
      clr_mode = 15;
     } else if (strcmp (charpnt, "-8") == 0) { /* 8-bit colour */
      clr_mode = 8;
     }
   }
 }

/* The bootstrap procedure of the program */
int main (int noparams, char *params []) {
 /* Subtract one from noparams to get correct number of parameters */
  phasecmdline (noparams, params);
  initialize ();
  gui_kernal (); /* Start the GUI kernel */

  return 0;
 }

/* Largely Advanced Body Animation Notation Author Toolkit for Outstanding Results

 Selectable Scrolling Directions
  * Traditional - Down to Up
  * Alternative - Up to Down

 Coloured Symbols
  * Unshaded          - Red
  * Dotted            - Yellow / Olive
  * Striped           - Green  / Dark Green
  * Filled            - Violet
  * Centre Stave Line - Dark Cyan
  * Outer Stave Line  - Cyan

    Column 0 - E

 */

/* _USE_LFN */
//#define _USE_LFN TRUE

