/*
 * LABANscript symbols
 *
 * Written and Copyright 1999 by David E. Down the Faery Monarch, Gonksoftware
 * This software is released under the GNU GPL version 2.
 *
 * See the file "COPYING" for software license details.
 */

#include "def.h"
#include "gui.h"
#include "mabanol.h"
#include "labanscr.h"

/*
The LABAN glyphs for Rudolf von Laban
=====================================

 0 (Empty crater for the pin)

    1    2       3   4       5   6
                 _   _
                | | | |
               _| | | |_
              |   | |   |
        |\    |   | |   |    /|
        |  \  |   | |   |  /  |
        |   | |___| |___| |   |
        |   |     ___     |   |
     /| |   |    |   |    |   | |\
   /  | |___|    |   |    |___| |  \
 /    |          |   |          |    \   Centre Symbol is 7
 \    |  ___     |   |     ___  |    /
   \  | |   |    |   |    |   | |  /
     \| |   |    |___|    |   | |/
        |   |  ___   ___  |   |
        |   | |   | |   | |   |          /\
        |  /  |   | |   |  \  |         /  \
        |/    |   | |   |    \|        /    \
              |_  | |  _|              \    /
                | | | |                 \  /
                |_| |_|                  \/

         8       9   B       C           A (not implemented)

 Other Glyphs
                                                          \        /
                                                            \    /
    /|  |\        /|  |\     |\ /|   / \   |\ /|   / \      /|  |\
  /  |  |  \    /  |  |  \   |   |  |   |  |   |  |   |   /  |  |  \
 |   |  |   |  |   |  |   |  |   |  |   |  |   |  |   |  |   |  |   |
 |   |  |   |  |   |  |   |  |   |  |   |  |   |  |   |  |   |  |   |
 |  /    \  |   \  |  |  /   |   |  |   |  |   |  |   |  |  /    \  |
 |/        \|     \|  |/     |/ \|   \ /    \ /   |/ \|  |/        \|
                                                          \        /
                                                            \    /
   D      E      F     10     11     12     13     14     15     16

    /    \           /       \       \         /       /     \
  /        \       /           \       \     /       /         \
 |\        /|     |\           /|   |\ /|   |\ /|   |\ /|   |\ /|
 |  \    /  |     |  \       /  |   |/  |   |  \|   |/  |   |  \|
 |   |  |   |     |    \   /    |   |   |   |   |   |   |   |   |
 |   |  |   |     |    /   \    |   |   |   |   |   |   |   |   |
 |  /    \  |     |  /       \  |   |  /|   |\  |   |  /|   |\  |
 |/        \|     |/           \|   |/ \|   |/ \|   |/ \|   |/ \|
  \        /       \           /     \         /       /     \
    \    /           \       /         \     /       /         \
  17     18          19      1A       1B     1C      1D      1E
                 (All symbols 19 - 2F are  not Implemented)


 SHADE TYPES:
 ============
 0 = No symbol
 1 = Dot
 2 = (reserved)
 3 = (reserved)
 4 = striped
 5 = filled

*/

#include "symdata.h"  /* Include the gylph data */

/* Draw the polyglyphs */
 VECTOR vectcoords[20]; /* Buffer */

 void vectors (BITMAP *bitmap, VECTOR *vector, int tx, int ty,
               int sx, int sy, UBYTE param, UBYTE rotate,
               LABAN_STRUCT *labandata) {
  int i = 0, k = 0, dx, dy, x, y, pt;

 /* Convert the coords, apply rotation if required */
  static inline void conversion (int ssx, int ssy, int *ttx, int *tty) {
   int tmp;

   if (rotate & 0x40) { tmp = ssx; ssx = ssy; ssy = tmp; }    /* Left Turn */
   if (rotate & 0x80) { ssx = 1000 - ssx; ssy = 1000 - ssy; } /* 180 Turn */
   *ttx = tx + (ssx * (sx - 1) / 1000); /* I could use ttx[0] instead */
   *tty = ty + (ssy * (sy - 1) / 1000); /* I could use tty[0] instead */
  }

 /* Feed the coords from vector data array, then convert them */
  static inline void convert (int *ttx, int *tty) {
   int ssx = vector[i++], ssy = vector[i++];

   conversion(ssx, ssy, ttx, tty);
  }

 /* Draw a continuous line */
  void drawline (int color) {
   pt = vector[i++]; /* Get the number of points */
     for (k = 0; k < pt; k++) {
      convert (&vectcoords[k << 1], &vectcoords[(k << 1) + 1]);
     }
   /* Draw the outline */
    pt--;
     for (k = 0; k < pt; k++) {
       line (bitmap, vectcoords[k << 1], vectcoords[(k << 1)+1],
             vectcoords[(k << 1)+2], vectcoords[(k << 1)+3], color);
      }
  }

  if (!vector) return; /* Do not sketch if NULL */
   while (vector[i] != VECT_END) {
    switch (vector[i++]) {
     case SS_LINE:   { /* Spacing glyph lines */
      drawline (labandata->hs_labanspace);
      break; }
     case SS_DOT:    { /* Space glyph dots (depends on param) */
      convert(&dx, &dy);
       if (param & 0x01) {
        putpixel (bitmap, dx, dy, labandata->hs_labanspace);
       }
      convert(&dx, &dy);
       if (param & 0x02) {
        putpixel (bitmap, dx, dy, labandata->hs_labanspace);
       }
      break; }
     case VECT_LINE: { /* Draw a series of lines (for polyglyphs) */
      drawline (labandata->hs_labanplyol);
      break; }
     case VECT_POLYGON: { /* Draw a polygon (for polyglyph) */
      pt = vector[i++]; /* Get the number of points */
      dx = vector[i++]; dy = vector[i++]; /* Get dot point */
        for (k = 0; k < pt; k++) {
         convert (&vectcoords [k << 1], &vectcoords [(k << 1) + 1]);
        }
       if (param == 5) { /* Draw completely shaded */
        polygon (bitmap, pt, vectcoords, labandata->hs_labanplyol);
       } else {
        polygon (bitmap, pt, vectcoords, labandata->hs_labanpysnd);
         if (param == 4) { /* Stripe */
          drawing_mode (DRAW_MODE_MASKED_PATTERN, laban_hash, 0, 0);
          polygon (bitmap, pt, vectcoords, labandata->hs_labanpymrk);
          solid_mode ();
         } else if (param == 2) { /* The X */
          conversion (dx, dy, &x, &y); /* Find the center */
          k = (sx / 3); x -= k >> 1; y -= k >> 1;
          if (k > 0) k--;
          line(bitmap, x, y, x + k, y + k, labandata->hs_labanpymrk);
          line(bitmap, x + k, y, x, y + k, labandata->hs_labanpymrk);
         } else if (param == 1) { /* Dot */
          conversion (dx, dy, &x, &y); /* Find the center */
          k = (sx / 5); x -= k >> 1; y -= k >> 1;
          if (k > 0) k--;
          rect(bitmap, x, y, x + k, y + k, labandata->hs_labanpymrk);
         }
       }
      /* Draw the outline */
      pt--;
      for (k = 0; k < pt; k++) {
        line (bitmap, vectcoords[k << 1], vectcoords[(k << 1)+1],
              vectcoords[(k << 1)+2], vectcoords[(k << 1)+3],
              labandata->hs_labanplyol);
       }
      line(bitmap, vectcoords[0], vectcoords[1], vectcoords[pt << 1],
           vectcoords[(pt << 1) + 1], labandata->hs_labanplyol);
      break; }
    }
   }
 }

/* The crater polygon glpyh */
 void gylph_pincrater (BITMAP *bitmap, int tx, int ty, int sx, int sy, LABAN_STRUCT *labandata) {
  drawing_mode (DRAW_MODE_MASKED_PATTERN, laban_dline, 0, 0);
  rect(bitmap, tx + 2, ty + 2, tx + sx - 3, ty + sy - 3, labandata->hs_labancrate);
  solid_mode ();
 }

/* Draw the glyph */
 void draw_gylph (BITMAP *bitmap, int tx, int ty, int bx, int by,
                  LABAN_PANTOMIME_SYMBOL symbol,
                  LABAN_PANTOMIME_SHADER shade,
                  LABAN_PANTOMIME_MODIFY modify,
                  LABAN_STRUCT *labandata) {
  int sx, sy, swap;

 /* Make sure that tx and ty are at top-left, and bx, by are bottom-right */
  if (tx > bx) { swap = tx; tx = bx; bx = swap; }
  if (ty > by) { swap = ty; ty = by; by = swap; }
  sx = bx - tx + 1; /* Work out the size */
  sy = by - ty + 1; /* Work out the size */

   if (symbol == 0x00) {       /* A pin crater */
    gylph_pincrater (bitmap, tx, ty, sx, sy - 1, labandata);
   } else if (symbol < 0x28) { /* A polyglyph */
    vectors (bitmap, polyglyph_array [symbol], tx, ty + 1, sx, sy - 2, shade, 0, labandata);
   } else if (symbol < 0x30) { /* === A non-polygon vectical glyph === */
   } else if (symbol = 0x30) { /* === Spacing Glyph === */
     if (shade & 0x10) { /* Rotateable glyph */
      vectors (bitmap, (shade & 0x80 ?
               ss_array_double [(shade & 0x7c) >> 2] :
               ss_array_single [(shade & 0x7c) >> 2]), tx, ty, sx, sy - 1,
               (shade & 0x3), modify, labandata);
     } else {           /* Lines-at-sides glyph */
      vectors (bitmap, (shade & 0x80 ?
               ss_array_double [(shade & 0x7c) >> 2] :
               ss_array_single [(shade & 0x7c) >> 2]), tx, ty, sx, sy - 1,
               (shade & 0x3), 0, labandata);
     }
   } /* if (symbol = ... */

 }

/*
 * LABAN based kinetographic tool by Gonksoftware.
 */

